//+------------------------------------------------------------------+
//|                                         VininI_VisualKohonen.mq4 |
//|                                         Copyright  2011, Vinin. |
//|                                                    vinin@mail.ru |
//+------------------------------------------------------------------+
#property copyright "Copyright  2011, Vinin."
#property link      "vinin@mail.ru"

#property show_inputs

extern int Width_NNN=32;
extern int HeightNNN=32;
extern int SizeInput= 3;
extern string NNNFileName="NNN";

extern int Epoch=32;
extern double R=0.9;
extern double Delta=0.1;

extern int FontSize= 8;




double NNN[];
double Input[];

string FileName;
//+------------------------------------------------------------------+
//| script program start function                                    |
//+------------------------------------------------------------------+
int start()
  {
   Koh.init();
   
   Koh.study();
   Visual.Draw();

   Koh.deinit();
   return(0);
  }
//+------------------------------------------------------------------+


void Koh.init(){
   FileName=NNNFileName+"_"+Width_NNN+"_"+HeightNNN+"_"+SizeInput+".csv";
   int N=Width_NNN*HeightNNN*SizeInput;
   ArrayResize(NNN, N);
   
   int Handle=FileOpen(FileName, FILE_READ | FILE_CSV);
   if (Handle==-1) {
      MathSrand(TimeLocal());
      for (int i=0;i<N;i++) NNN[i]=(2.0*MathRand()-32767.0)/32767.0;
   
   } else {
      for (i=0;i<N;i++) NNN[i]=FileReadNumber(Handle);
      FileClose(Handle);

   }
   Visual.init();
}

void Koh.deinit(){
   int N=Width_NNN*HeightNNN*SizeInput;
   int Handle=FileOpen(FileName, FILE_WRITE | FILE_CSV);
   if (Handle>=0) {
      for (int i=0;i<N;i++) FileWrite(Handle, NNN[i]);
      FileClose(Handle);
   }
   Visual.deinit();
}

int pos[2];

void Koh.Work(double Input[]){
   double Error=999999.9999;
   for (int i=0;i<Width_NNN;i++) {
      for (int j=0;j<HeightNNN;j++) {
         int n=Index(i,j);
         double tmp=0;
         for (int k=0;k<SizeInput;k++)  tmp+=((Input[k]-NNN[n+k])*(Input[k]-NNN[n+k]));

         if (Error<tmp) continue;
         Error=tmp;
         pos[0]=i;
         pos[1]=j;
      }
   }
}

int PrevPos[]={-1,-1};

double max=0.00001, min=-0.00001;

void Koh.study(){
   double Input[];
   ArrayResize(Input, SizeInput);
   double r, delta;
   for (int ep=0;ep<Epoch;ep++) {
      r=R*MathMax(Width_NNN,HeightNNN)*(1.0-ep/Epoch);      
      delta=Delta*(1.0-ep/Epoch);
      for (int i=1;i<Bars-SizeInput*7;i++) {
         for (int j=0;j<SizeInput;j++) {
            Input[j]=iAO(NULL,0,i+j*7);
            if (max>Input[j]) max=Input[j];
            if (min<Input[j]) min=Input[j];
         }
         Koh.Work(Input);
         if (PrevPos[0]==pos[0] && PrevPos[1]==pos[1]) continue;
         PrevPos[0]=pos[0];
         PrevPos[1]=pos[1];
         Neuron.study(Input, r, delta);
         Visual.Draw();
      }
   }
}




void Neuron.study(double Input[], double lR, double lDelta){
   int i, j, k, I, J, IMin, IMax, JMin, JMax;
   
   if (lR>1) {

      IMin=MathMax(0,(pos[0]-lR));
      IMax=MathMin(Width_NNN,(pos[0]+lR));
      JMin=MathMax(0,(pos[1]-lR));
      JMax=MathMin(HeightNNN,(pos[1]+lR));
      for (i=IMin;i<=IMax;i++) {
         for (j=JMin;j<=JMax;j++) {
            double r=MathSqrt((pos[0]-i)*(pos[0]-i)+(pos[1]-j)*(pos[1]-j));
            if (r<=lR) {
               for (k=0;k<SizeInput;k++) {
                  int n=Index(i,j,k);
                  NNN[n]=NNN[n]+(Input[k]-NNN[n])*lDelta*(1.0-r/lR);
               }
            }
         }
      }
   } else {
      n=Index(pos[0],pos[1]);
      for (k=0;k<SizeInput;k++) {
         NNN[n+k]=NNN[n+k]+(Input[k]-NNN[n+k])*lDelta;
      }
   }
   
}



color colorMax;
   


void Visual.init(){
   colorMax=255;
   if (SizeInput>3) colorMax=MathCeil(MathPow(255.0*255.0*255.0,1.0/SizeInput));

}

void Visual.deinit(){


}

void Visual.Draw(){
   max=MathMax(NNN[ArrayMaximum(NNN,WHOLE_ARRAY,0)],max);
   min=MathMin(NNN[ArrayMinimum(NNN,WHOLE_ARRAY,0)],min);
   string Name="NNN";
   for (int i=0;i<Width_NNN;i++) {
      for (int j=0;j<HeightNNN;j++) {
         Neuron.Draw(Name+i+"_"+j, i, j, Neuron.Color(i, j));
      }
   }
}

color Neuron.Color(int i, int j) {
   color Res=0x00000000;
   int K=Index(i,j);
   for (int k=0;k<SizeInput;k++) {
      Res*=colorMax;
      Res+=MathCeil((NNN[K+k]-min)/(max-min)*colorMax);
   }
   return(Res);
}

void Neuron.Draw(string Name, int i, int j, color cl){
   DrawObject(Name, StringSetChar("",0,103), FontSize, "Webdings", i*FontSize*1.5, (j+1)*FontSize*1.5, cl);
}


void DrawObject(string Name, string Text, int Size, string Font, int X, int Y, color CLR, bool bBack=false,int Angle=0){
   if (ObjectFind(Name)==-1)  ObjectCreate(Name, OBJ_LABEL, 0, 0,0,0,0);
   ObjectSet(Name, OBJPROP_CORNER,     0     );
   ObjectSet(Name, OBJPROP_XDISTANCE,  X     );
   ObjectSet(Name, OBJPROP_YDISTANCE,  Y     );
   ObjectSet(Name, OBJPROP_BACK,       bBack );
   ObjectSet(Name, OBJPROP_ANGLE,      Angle );
   ObjectSetText(Name,Text, Size, Font, CLR  );
}

int Index(int w, int h, int g=0){return((w*HeightNNN+h)*SizeInput+g);}

int IndexW(int pos){
   int Res=pos;
   Res/=SizeInput;
   Res/=HeightNNN;
   return(Res);
}

